// DataMirror.h
// Memory and register content mirroring for access by the 
// the Graphical UI an the Graphical visualiser.
// Graphical visualiser for the JET language.
// A. Savidis, Y.Lilis.
// November, 2005.
// Project: JET Suite (Execution visualizer, Savidis, 2005).
//

#if	defined(HAS_VISUALIZER) && defined(GRAPHICAL_VISUALIZER)

#ifndef	DATAMIRROR_H
#define	DATAMIRROR_H

#pragma warning(disable:4786)

#include <map>
#include <string>

class DataMirror {

	friend class GraphicalVisualiser;

	private:
	////////////////////////////////////////////////////////

	static DataMirror* singletonPtr;
	static void	SingletonCreate (void);
	static void	SingletonDestroy (void);

	//**********************

	typedef std::map<unsigned, int> CellMap;
	CellMap		memory;
	CellMap		registers;

	unsigned	MemMutex;
	unsigned	RegMutex;

	int* GetCell (CellMap& Map, unsigned index);

	int* NewCell (CellMap& Map, unsigned index);
	
	////////////////////////////////////////////////////////

	public:
	static DataMirror*	GetSingleton (void);

	//**********************

#define FUNCTION_PROTOTYPES(_type)\
	void	Grant##_type##Access	(void);\
	void	Release##_type##Access	(void);\
	\
	bool	Is##_type##Occupied		(unsigned index);\
	\
	int		Get##_type##Content		(unsigned index);\
	\
	void	Set##_type##Content		(unsigned index, int value);

	FUNCTION_PROTOTYPES(Mem)
	FUNCTION_PROTOTYPES(Reg)

	//**********************

	DataMirror (void);
	~DataMirror();
};

#endif	// Header file end point.

#endif	// HAS_VISUALIZER && GRAPHICAL_VISUALIZER

