// ExecutionEvents.h
// Place holder for the interceptable JET execution events,
// encompassing all necessary callbacks, providing the
// typical "set" and "notification" functionality.
// A. Savidis, Y.Lilis.
// November, 2005.
// Project: JET Suite (Execution visualizer, Savidis, 2005).
//

#ifdef	HAS_VISUALIZER

#ifndef	EXECUTIONEVENTS_H

#pragma warning(disable:4786)

#include <stdio.h>
#include <string>
using namespace std;

///////////////////////////////////////////////////////////////////

class ExecutionEvents {

	private:

	static void (*onRegAssign)	(unsigned reg, int value);
	static void (*onMemAssign)	(unsigned MemCell, int value);
	
	//*************************
	
	static void (*onRFormat)	(const char* name, unsigned ra, unsigned rb, unsigned rc, unsigned line);
	static void (*onIFormat)	(const char* name, unsigned ra, unsigned rb, int imm13, unsigned line);
	static void (*onLFormat)	(const char* name, unsigned ra, int imm19, unsigned line);
	
	//*************************
	
	static void (*onBranch)		(const char* name, unsigned char mask, unsigned ra, unsigned rb, const char* label, unsigned line);
	static void (*onJump)		(const char* name, const char* label, unsigned line);

	//*************************

	static void (*onProgramStart)	(void);
	static void (*onProgramEnd)		(void);
	static void (*onProgramError)	(const char* s);

	//*************************

	static int			(*outputFun)	(const char *format, ...);
	static const char*	(*inputFun)		(const char* buffer, unsigned len, FILE* fp);
	static void			(*cleanUpFun)	(void);

	///////////////////////////////////////////////////////////////////

	public:

	static void SetOnRegAssign (void (*f)(unsigned reg, int value))
					{  onRegAssign = f; }

	static void SetOnMemAssign (void (*f)(unsigned MemCell, int value))
					{  onMemAssign = f; }

	//*************************

	static void SetOnRFormat (void (*f)(const char* name, unsigned ra, unsigned rb, unsigned rc, unsigned line))
					{ onRFormat = f; }
	
	static void SetOnIFormat (void (*f)(const char* name, unsigned ra, unsigned rb, int imm13, unsigned line))
					{ onIFormat = f;}

	static void SetOnLFormat (void (*f)(const char* name, unsigned ra, int imm19, unsigned line))
					{ onLFormat = f;}

	//*************************

	static void SetOnBranch (void (*f)(const char* name, unsigned char mask, unsigned ra, unsigned rb, const char* label, unsigned line))
					{ onBranch = f; }

	static void SetOnJump (void (*f)(const char* name, const char* label, unsigned line))
					{ onJump = f; }

	///////////////////////////////////////////////////////////////////

	static void NotifyRegAssign (unsigned reg, int value)
					{  (*onRegAssign)(reg, value); }

	static void NotifyMemAssign (unsigned MemCell, int value)
					{  (*onMemAssign)(MemCell, value); }

	//*************************

	static void NotifyRFormat (const char* name, unsigned ra, unsigned rb, unsigned rc, unsigned line)
					{ (*onRFormat)(name, ra, rb, rc, line); }
	
	static void NotifyIFormat (const char* name, unsigned ra, unsigned rb, int imm13, unsigned line)
					{ (*onIFormat)(name, ra, rb, imm13, line);}

	static void NotifyLFormat (const char* name, unsigned ra, int imm19, unsigned line)
					{ (*onLFormat)(name, ra, imm19, line);}

	//*************************

	static void NotifyBranch (const char* name, unsigned char mask, unsigned ra, unsigned rb, const char* label, unsigned line)
					{ (*onBranch)(name, mask, ra, rb, label, line); }

	static void NotifyJump (const char* name, const char* label, unsigned line)
					{ (*onJump)(name, label, line); }


#define NOTIFY_REGASSIGN(r, c)				ExecutionEvents::NotifyRegAssign(NUM_OF_REG(*this), content)
#define NOTIFY_MEMASSIGN(m, c)				ExecutionEvents::NotifyMemAssign(jet.GetMemCellNo(this), x)
#define NOTIFY_RFORMAT(n, ra, rb, rc, l)	ExecutionEvents::NotifyRFormat(n, ra, rb, rc, l)
#define NOTIFY_IFORMAT(n, ra, rb, im, l)	ExecutionEvents::NotifyIFormat(n, ra, rb, im, l)
#define NOTIFY_LFORMAT(n, ra, im, l)		ExecutionEvents::NotifyLFormat(n, ra, im, l)
#define NOTIFY_BRANCH(n, id, ra, rb, la, l)	ExecutionEvents::NotifyBranch(n, id, ra, rb, la, l)
#define NOTIFY_JUMP(n, la, l)				ExecutionEvents::NotifyJump(n, la, l)

	//*************************
	// PRORGAM RUN AND I/O.
	
	static void SetOnProgramStart (void (*f)(void)) { onProgramStart = f; }
	static void SetOnProgramEnd (void (*f)(void)) { onProgramEnd = f; }
	static void SetOnProgramError (void (*f)(const char* s)) { onProgramError = f; }

	static void NotifyProgramStart (void) { (*onProgramStart)(); }
	static void NotifyProgramEnd (void) { (*onProgramEnd)(); }
	static void NotifyProgramError (const char* s) { (*onProgramError)(s); }

#define	NOTIFY_PROGRAMSTART()		ExecutionEvents::NotifyProgramStart()
#define	NOTIFY_PROGRAMEND()			ExecutionEvents::NotifyProgramEnd()
#define	NOTIFY_PROGRAMERROR(s)		ExecutionEvents::NotifyProgramError(s)

	static void SetOutputFunction (int (*f)(const char *format, ...))
		{ outputFun = f; }

	static void SetInputFunction (const char* (*f)(const char *buf, unsigned len, FILE* fp))
		{ inputFun = f; }

	static void SetCleanUpFunction (void (*f)(void))
		{ cleanUpFun = f; }

	typedef int (*OutputFuncType)(const char *format, ...);
	typedef const char* (*InputFuncType)(const char *buff, unsigned len, FILE* fp);
	typedef void (*CleanUpFuncType) (void);

	static OutputFuncType GetOutputFunction (void) 
		{ return outputFun; }

	static InputFuncType GetInputFunction (void) 
		{ return inputFun; }

	static CleanUpFuncType GetCleanUpFunction (void) 
		{ return cleanUpFun; }

#define	INPUT_FUNCTION		ExecutionEvents::GetInputFunction()
#define	OUTPUT_FUNCTION		ExecutionEvents::GetOutputFunction()
#define	CLEANUP_FUNCTION	ExecutionEvents::GetCleanUpFunction()

};

#endif	// Dop not add stuff beyond this point.

#endif	// HAS_VISUALIZER
