// GRAPHICALUILinkage.h
// The linkage with the Graphical User Interface.
// It encompasses the API the UI has to implement.
// Graphical visualiser for the JET language.
// A. Savidis, Y.Lilis.
// November, 2005.
// Project: JET Suite (Execution visualizer, Savidis, 2005).
//


#if	defined(HAS_VISUALIZER) && defined(GRAPHICAL_VISUALIZER)

#pragma warning(disable:4786)

#ifdef	CONSOLE_VISUALISER
#error "Should define either GRAPHICAL_VISUALIZER or CONSOLE_VISUALIZER!"
#endif

#ifndef	GRAPHICALUILINKAGE_H
#define	GRAPHICALUILINKAGE_H

#include <string>

////////////////////////////////////////////////////////////
// The linkage to the UI which runs in parallel. In case
// of user quit from the UI, the UI has the privilege to call 
// exit.
//
namespace GRAPHICALUI {
extern void		Create	(void);								// Creates UI and thread. Ready to go.
extern void		Destroy (void);								// Destroyes UI and cleans-up the thread.
extern void		MemoryRefresh (unsigned index);				// Ask to refersh contents of a memory cell.
extern void		RegisterRefresh (unsigned reg);				// Ask to refersh contents of a register.

extern void		StepWait	(void);							// Wait until step is selected by the user.
extern void		InputWait	(char* buffer, unsigned len);	// Wait until input is put in this buffer.
extern void		Trace		(const char* msg);				// Simple trace message.
extern void		Output		(const char* msg);				// Normal program output.

extern void		Finished	(void);							// Implement on program finished actions.
extern void		Started		(void);							// Implement on program started actions.

extern unsigned CreateMutex		(void);						// Unique, non-zero.
extern void		DestroyMutex	(unsigned id);
extern void		GrantMutex		(unsigned id);
extern void		ReleaseMutex	(unsigned id);
}

////////////////////////////////////////////////////////////
// Those functions are already implemented at the gaphical
// visualiser side, so they have to be used for event control
// by the UI implementation.
//
namespace EVENTAPI {
extern bool IsNotifyingEvent	(const std::string& name);
extern void SetNotifyingEvent	(const std::string& name, bool enabled);
extern bool IsSteppingEvent		(const std::string& name);
extern void SetSteppingEvent	(const std::string& name, bool enabled);
}

#endif	// Header file end point.

#endif	// HAS_VISUALIZER && CONSOLE_VISUALIZER
