// GraphicalVisualiser.h
// A Graphical visualiser for the FLIP language.
// A. Savidis, Y.Lilis.
// November, 2005.
// Project: JET Suite (Execution visualizer, Savidis, 2005).
//

#ifndef	GRAPHICALVISUALISER_H
#define	GRAPHICALVISUALISER_H

#pragma warning(disable:4786)

#if	defined(HAS_VISUALIZER) && defined(GRAPHICAL_VISUALIZER)

#ifdef	CONSOLE_VISUALISER
#error "Should define either GRAPHICAL_VISUALIZER or CONSOLE_VISUALIZER!"
#endif

#include "VisualiserSuper.h"
#include <stdio.h>
#include <assert.h>

////////////////////////////////////////////////////////////

class GraphicalVisualiser : public VisualiserSuper {

	////////////////////////////////////////////////////////////

	private:
	static const char*	Input (const char* buff, unsigned len, FILE* fp);
	static int			Output (const char* format, ...);

	static void			CleanUp (void);

	static void			OnProgramStart (void);
	static void			OnProgramEnd (void);
	static void			OnProgramError (const char* s);

	//*****************************

	static void PreInitialiser (void);
	static void PostCleaner (void);
	
	unsigned eventMutex;

	////////////////////////////////////////////////////////////
	
	public:
	
	// Thread enabled verisons for the graphical visualiser that works in parallel.
	//
	virtual bool IsNotifyingEvent (const std::string& name);

	virtual void SetNotifyingEvent (const std::string& name, bool enabled);

	virtual bool IsSteppingEvent (const std::string& name);

	virtual void SetSteppingEvent (const std::string& name, bool enabled);

	//*****************************

	void Message (const char* str) const;

	void MemCellRefresh (unsigned MemCell, int value);

	void RegRefresh (unsigned reg, int value);

	void StepWait (void) const;

	GraphicalVisualiser (void);
	~GraphicalVisualiser();
};


///////////////////////////////////////////////////////

#endif	// Header file end point.

#endif	// HAS_VISUALIZER && CONSOLE_VISUALIZER
