// ConsoleVisualiser.cpp
// A console visualiser for the JET language.
// A. Savidis, Y.Lilis.
// November, 2005.
// Project: JET Suite (Execution visualizer, Savidis, 2005).
//

#if	defined(HAS_VISUALIZER) && defined(CONSOLE_VISUALIZER)

#include "ConsoleVisualiser.h"
#include <stdio.h>
#include <stdarg.h>
#include <assert.h>

bool				ConsoleVisualiser::fileOutput;
FILE*				ConsoleVisualiser::file;

VISUALISER_LINKAGE(ConsoleVisualiser)

#define	CONSOLE_STDOUTPUT printf

////////////////////////////////////////////////////////////////
// Standard console I/O functions.
//
const char* ConsoleVisualiser::Input (const char* buff, unsigned len, FILE* fp) {
	return fgets(const_cast<char*>(buff), len, fp);
}

int ConsoleVisualiser::Output (const char* format, ...) {

	va_list args;
	va_start(args, format);

	int result;
	static char buffer[1024];
	result = vsprintf(buffer, format, args);

	if (fileOutput) {
		assert(file);
		fprintf(file, "%s", buffer);
	}
	else {
		printf("%s", buffer);
		fflush(stdout);
	}

	va_end(args);
	return result;
}

////////////////////////////////////////////////////////////////

void ConsoleVisualiser::CleanUp (void) {

	assert(!cleanedUp && visualiser);

	cleanedUp = true;
	delete ((ConsoleVisualiser*) visualiser);	// Derived class specific.
	visualiser = (VisualiserSuper*) 0;

	Output("Console execution tracer closed.\n");

	PostCleaner();
	exit(0);									// Clean-up exits immediately!
}

//*****************************

void ConsoleVisualiser::OnProgramStart (void) {
	CONSOLE_STDOUTPUT("JET starts running the program.\n");
	Output("Console execution tracer activated.\n");
}

void ConsoleVisualiser::OnProgramEnd (void) {
	CONSOLE_STDOUTPUT("JET finished the program normally.\n");
	CleanUp();
}

void ConsoleVisualiser::OnProgramError (const char* s) {
	CONSOLE_STDOUTPUT("Error: %s: JET has interrupted program execution.\n", s);
}

//*****************************

ConsoleVisualiser::ConsoleVisualiser (void) {

	PreInitialiser();

	ExecutionEvents::SetInputFunction(Input);
	ExecutionEvents::SetOutputFunction(CONSOLE_STDOUTPUT);
	ExecutionEvents::SetCleanUpFunction(CleanUp);

	ExecutionEvents::SetOnProgramStart(OnProgramStart);
	ExecutionEvents::SetOnProgramEnd(OnProgramEnd);
	ExecutionEvents::SetOnProgramError(OnProgramError);
}

ConsoleVisualiser::~ConsoleVisualiser() {
}

////////////////////////////////////////////////////////////////

void ConsoleVisualiser::Message (const char* str) const{
	Output("%s.\n", str);
}

//*****************************

void ConsoleVisualiser::MemCellRefresh (unsigned MemCell, int value) {
	assert(visualiser);

	Message(
		MakeString(
			"Memory assignment : MEM[%u] = %d", 
			MemCell,
			value
		).c_str()
	);
}

//*****************************

void ConsoleVisualiser::RegRefresh (unsigned reg, int value) {
	assert(visualiser);

	Message(
		MakeString(
			"Register assignment : REG[%u] = %d",
			reg,
			value
		).c_str()
	);
}

////////////////////////////////////////////////////////////////

#endif	// HAS_VISUALIZER && CONSOLE_VISUALIZER
