// DataMirror.cpp
// Memory and register content READ access for the GTK UI.
// GTK visualiser for the JET language.
// A. Savidis, Y.Lilis.
// November, 2005.
// Project: JET Suite (Execution visualizer, Savidis, 2005).
//

#if	defined(HAS_VISUALIZER) && defined(GRAPHICAL_VISUALIZER)

#include <assert.h>
#include "GraphicalUILinkage.h"
#include "DataMirror.h"

////////////////////////////////////////////////////////////////

DataMirror* DataMirror::singletonPtr;

void DataMirror::SingletonCreate (void) {
	assert(!singletonPtr);
	singletonPtr = new DataMirror;
}

void DataMirror::SingletonDestroy (void) {
	assert(singletonPtr);
	delete singletonPtr;
	singletonPtr = (DataMirror*) 0;
}

DataMirror*	DataMirror::GetSingleton (void) {
	assert(singletonPtr);
	return singletonPtr;
}

//************************

DataMirror::DataMirror (void) {
	MemMutex  = GRAPHICALUI::CreateMutex();
	RegMutex  = GRAPHICALUI::CreateMutex();
	assert(MemMutex);
	assert(RegMutex);
}

DataMirror::~DataMirror() {
	GRAPHICALUI::DestroyMutex(MemMutex);
	GRAPHICALUI::DestroyMutex(RegMutex);
	MemMutex = 0;
	RegMutex = 0;
	memory.clear();
	registers.clear();
}

////////////////////////////////////////////////////////////////

int* DataMirror::GetCell (CellMap& Map, unsigned index) {
	CellMap::iterator i = Map.find(index);
	return i == Map.end() ? (int*) 0 : &i->second;
}

int* DataMirror::NewCell (CellMap& Map, unsigned index) {
	assert(!GetCell(Map, index));
	Map[index] = 0;
	return GetCell(Map, index);
}

#define FUNCTION_BODIES(_type, _map)\
	void DataMirror::Grant##_type##Access (void) {\
		GRAPHICALUI::GrantMutex(_type##Mutex);\
	}\
	void DataMirror::Release##_type##Access (void) {\
		GRAPHICALUI::ReleaseMutex(_type##Mutex);\
	}\
	\
	bool DataMirror::Is##_type##Occupied (unsigned index) {\
		return !!GetCell(_map, index);\
	}\
	\
	int DataMirror::Get##_type##Content (unsigned index) {\
		int* cell = GetCell(_map, index);\
		assert(cell);\
		return *cell;\
	}\
	\
	void  DataMirror::Set##_type##Content (unsigned index, int value) {\
		int* p = GetCell(_map, index);\
		if (!p)\
			p = NewCell(_map, index);\
		assert(p);\
		*p = value;\
	}

	FUNCTION_BODIES(Mem, memory)
	FUNCTION_BODIES(Reg, registers)

////////////////////////////////////////////////////////////////

#endif	// HAS_VISUALIZER && GRAPHICAL_VISUALIZER
